""" Common config across environments """


def plugin_settings(settings):
    """
    App-specific settings
    """

    # Available languages for translation
    settings.AI_TRANSLATIONS_LANGUAGE_CONFIG = {
        "en": "English",
        "es": "Español",
        "ar": "العربية",
    }

    # TTL in seconds for successful request to store in cache
    settings.REQUEST_CACHE_SUCCESS_TIMEOUT = 86400

    # TIL in seconds for failure request to store in cache
    settings.REQUEST_CACHE_FAILURE_TIMEOUT = 300

    # Timeout in seconds for API requests
    settings.AI_TRANSLATIONS_API_TIMEOUT = 15

    # Provider type for AI transcripts
    # See ref: https://github.com/openedx/edx-val/blob/b7c5db6c95c9a8655e1f3d58429c2e52b4ead66e/edxval/models.py#L381
    settings.AI_TRANSCRIPTS_PROVIDER_TYPE = 'edx_ai_translations'



"""Tests for translatable_xblocks/util.py"""

from unittest import TestCase
from unittest.mock import patch

import ddt
from lxml import etree

from translatable_xblocks.utils import (
    convert_html_to_xml,
    language_selection_to_django_language,
    language_selection_to_django_locale,
    reinsert_base64_images,
    replace_img_base64_with_placeholder,
)


@ddt.ddt
class TestTranslationLanguageToDjangoLocale(TestCase):
    """Test for language_selection_to_django_local"""

    @ddt.unpack
    @ddt.data(
        ["en", "en"],
        ["ab-cd", "ab_CD"],  # Locale uses underscore and capitalized locale
    )
    def test_language_selection_to_django_locale(self, language_code, expected_locale):
        # Given a valid language code
        valid_language_code = language_code

        # When I ask for its correlated Django locale
        django_locale = language_selection_to_django_locale(valid_language_code)

        # Then I get the correct Django locale
        self.assertEqual(expected_locale, django_locale)

    @ddt.unpack
    @ddt.data(
        [
            "es",
            "es_419",
        ]  # Google only uses "es" for Spanish, but Django further differentiates by region
    )
    def test_language_selection_to_django_locale_special_cases(
        self, language_code, expected_locale
    ):
        # Given a language code that requires special mapping
        special_case_language_code = language_code

        # When I ask for its correlated Django locale
        django_locale = language_selection_to_django_locale(special_case_language_code)

        # Then I get the correct Django locale
        self.assertEqual(expected_locale, django_locale)

    @ddt.data(None, 323, object)
    def test_language_selection_to_django_locale_bad_arg(self, language_code):
        # Given a bad language code
        bad_language_code = language_code

        # When I ask for its correlated Django-locale
        django_locale = language_selection_to_django_locale(bad_language_code)

        # Then I return None
        self.assertIsNone(django_locale)


@ddt.ddt
class TestTranslationLanguageToDjangoLanguage(TestCase):
    """Tests for language_selection_to_django_language"""

    @ddt.unpack
    @ddt.data(
        ["en", "en"],
        ["ab-cd", "ab-cd"],  # Language uses hyphenated region selector
    )
    def test_language_selection_to_django_language(
        self, language_code, expected_django_language
    ):
        # Given a valid language code
        valid_language_code = language_code

        # When I ask for its correlated Django locale
        django_language = language_selection_to_django_language(valid_language_code)

        # Then I get the correct Django language code
        self.assertEqual(expected_django_language, django_language)

    @ddt.unpack
    @ddt.data(
        ["es", "es-419"],
        ["ko", "ko-kr"],
        # Google uses a slightly different set of language specifiers than Django, which require mapping
    )
    def test_language_selection_to_django_language_special_cases(
        self, language_code, expected_django_language
    ):
        # Given a language code that requires special mapping
        special_case_language_code = language_code

        # When I ask for its correlated Django locale
        django_language = language_selection_to_django_language(
            special_case_language_code
        )

        # Then I get the correct Django language code
        self.assertEqual(expected_django_language, django_language)

    @ddt.data(None, 323, object)
    def test_language_selection_to_django_language_bad_arg(self, language_code):
        # Given a bad language code
        bad_language_code = language_code

        # When I ask for its correlated Django-locale
        django_language = language_selection_to_django_language(bad_language_code)

        # Then I return None
        self.assertIsNone(django_language)


class TestHtmlXmlConversion(TestCase):
    """Tests for AiTranslationService."""

    @patch("translatable_xblocks.utils.get_platform_etree_lib")
    def test_convert_html_to_xml(self, mock_platform_etree):
        # edx-platform enforces its own import of etree for safety...
        # but for testing, we can just use etree
        mock_platform_etree.return_value = etree

        # Given an HTML response from the Google Translate API
        original_xml = '<option correct="False">Attributes should be quoted</option>'
        html_wrapped_xml = "<html><body><option correct=False>Attributes should be quoted</option></body></html>"

        # When I try to convert it to XML
        xml_output = convert_html_to_xml(html_wrapped_xml)

        # Then I get the expected output
        self.assertEqual(original_xml, xml_output)


class TestBase64ImageReplacement(TestCase):
    """Tests for replacing base64 images with placeholders."""
    def test_base64_image_replace(self):
        text = (
            '<p>Here is an image:</p>'
            '<img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUA" alt="Example">'
            '<p>Another paragraph.</p>'
            '<img src="data:image/jpg;base64,iBlahBlahBlah" alt="Example2">'
        )

        expected_text = (
            '<p>Here is an image:</p><img src="BASE64_IMG_PLACEHOLDER_0" alt="Example">'
            '<p>Another paragraph.</p><img src="BASE64_IMG_PLACEHOLDER_1" alt="Example2">'
        )

        replaced_text, base64_images = replace_img_base64_with_placeholder(text)

        self.assertEqual(expected_text, replaced_text)
        self.assertEqual(len(base64_images), 2)

        actual_text = reinsert_base64_images(replaced_text, base64_images)
        self.assertEqual(actual_text, text)



"""
Utilities used across the Translatable XBlocks Repo.
"""
import re

from django.utils.translation import to_language, to_locale

from translatable_xblocks.platform_imports import get_platform_etree_lib

# The Google and Django languages / locales differ. Specifically, while Django includes
# extra region-specific specifiers like Spanish (Spain) vs Spanish (Latin America),
# Google does not. This helps map our requested translation (for Google) to Django.
GOOGLE_TO_DJANGO_LANGUAGE_CODE_MAPPER = {
    "es": "es-419",
    "ko": "ko-kr",
}


def language_selection_to_django_locale(language):
    """Convert requested translation language to Django locale code, if provided."""
    if isinstance(language, str):
        return to_locale(language_selection_to_django_language(language))
    return None


def language_selection_to_django_language(language):
    """Convert requested tranlsation language to Django language code."""
    if isinstance(language, str):
        if language in GOOGLE_TO_DJANGO_LANGUAGE_CODE_MAPPER:
            return GOOGLE_TO_DJANGO_LANGUAGE_CODE_MAPPER[language]
        return to_language(language)
    return None


def convert_html_to_xml(html_style_xml):
    """
    Google does not natively handle translating of XML but does handle HTML.

    For blocks with XML definitions (e.g. problem blocks)

    For translations of blocks with XML definitions (e.g. Problem Block) we have to
    use the HTML translation mode of the Google Translate API. This is the closest we can
    currently get without native XML translation support.

    A "helpful" feature of this though is that the API does a fair amount of
    reformatting. For HTML, they remove invalid tag structures (like any tag inside
    of a <label>) as well as unquote single word attributes in tags. Our XML parser
    fails for unquoted attributes so this feature breaks several problem types.

    The workaround is to parse the translated XML as HTML and rewrite to string. The
    pair of parsers here re-quote the attributes we need quoted with the side effect
    of also wrapping the entire problem in HTML / Body tags, which we strip before
    returning.
    """
    # By parsing as HTML instead of XML, we can read translated data without error
    etree = get_platform_etree_lib()
    parser = etree.HTMLParser()
    wrapped_problem_xml_tree = etree.fromstring(html_style_xml, parser)

    # Unpack problem from automatically-added HTML and Body tags
    problem_xml_tree = wrapped_problem_xml_tree[0][0]
    problem_xml = etree.tostring(problem_xml_tree, encoding="unicode")

    return problem_xml


def replace_img_base64_with_placeholder(content):
    """
    Replace base64 images with placeholder.

    This is needed to prevent the code from sending huge amount of text to Google
    as base64 images are usually single long strings that exceeds Google translate limit.
    """
    base64_images = []
    placeholder_template = "BASE64_IMG_PLACEHOLDER_{index}"

    # Function to replace base64 src with placeholders
    def replace_match(match):
        base64_images.append((match.group(3), match.group(4)))
        placeholder = placeholder_template.format(index=len(base64_images) - 1)
        return f'<img {match.group(1)}src="{placeholder}"'

    # Regex to find <img> tags with base64 src
    img_tag_regex = r'<img\s+([^>]*?)src=(["\'])data:image/(png|jpeg|jpg|gif);base64,([^"\']+)\2'
    processed_content = re.sub(img_tag_regex, replace_match, content)

    return processed_content, base64_images


def reinsert_base64_images(content, base64_images):
    """Replace placeholders with the original base64 string."""
    for index, (image_type, base64_str) in enumerate(base64_images):
        placeholder = f"BASE64_IMG_PLACEHOLDER_{index}"
        content = content.replace(placeholder, f"data:image/{image_type};base64,{base64_str}")
    return content
